[<prev] [next>] [day] [month] [year] [list]
Message-ID: <20250131195639.1784933-4-ftreven@opensource.cirrus.com>
Date: Fri, 31 Jan 2025 13:56:34 -0600
From: Fred Treven <ftreven@...nsource.cirrus.com>
To: Simon Trimmer <simont@...nsource.cirrus.com>,
Charles Keepax
<ckeepax@...nsource.cirrus.com>,
Richard Fitzgerald
<rf@...nsource.cirrus.com>
CC: Fred Treven <ftreven@...nsource.cirrus.com>,
<patches@...nsource.cirrus.com>, <linux-kernel@...r.kernel.org>
Subject: [PATCH 3/7] firmware: cs_dsp: Add ability to load multiple coefficient files
Add cs_dsp_power_up_multiple() which accepts an array of
cs_dsp_coeff_desc firmware-filename pairs to load.
This enables the user to load more than one tuning file
along with the associated firmware.
Change cs_dsp_power_up() to make use of the new function
with a single coefficient file.
Signed-off-by: Fred Treven <ftreven@...nsource.cirrus.com>
---
drivers/firmware/cirrus/cs_dsp.c | 61 ++++++++++++++++++++------
include/linux/firmware/cirrus/cs_dsp.h | 14 ++++++
2 files changed, 62 insertions(+), 13 deletions(-)
diff --git a/include/linux/firmware/cirrus/cs_dsp.h b/include/linux/firmware/cirrus/cs_dsp.h
index 7cae703b3137..4c4e746be6fa 100644
--- a/include/linux/firmware/cirrus/cs_dsp.h
+++ b/include/linux/firmware/cirrus/cs_dsp.h
@@ -52,6 +52,16 @@
#define CS_DSP_WSEQ_UNLOCK 0xFD
#define CS_DSP_WSEQ_END 0xFF
+/**
+ * struct cs_dsp_coeff_desc - Describes a coeff. file + filename pair
+ * @coeff_firmware: Firmware struct to populate with coeff. data
+ * @coeff_filename: File from which coeff. data is loaded
+ */
+struct cs_dsp_coeff_desc {
+ const struct firmware *coeff_firmware;
+ const char *coeff_filename;
+};
+
/**
* struct cs_dsp_region - Describes a logical memory region in DSP address space
* @type: Memory region type
@@ -227,6 +237,10 @@ int cs_dsp_adsp1_power_up(struct cs_dsp *dsp,
const struct firmware *coeff_firmware, const char *coeff_filename,
const char *fw_name);
void cs_dsp_adsp1_power_down(struct cs_dsp *dsp);
+int cs_dsp_power_up_multiple(struct cs_dsp *dsp,
+ const struct firmware *wmfw_firmware, const char *wmfw_filename,
+ struct cs_dsp_coeff_desc *coeffs, int num_coeffs,
+ const char *fw_name);
int cs_dsp_power_up(struct cs_dsp *dsp,
const struct firmware *wmfw_firmware, const char *wmfw_filename,
const struct firmware *coeff_firmware, const char *coeff_filename,
diff --git a/drivers/firmware/cirrus/cs_dsp.c b/drivers/firmware/cirrus/cs_dsp.c
index aacf6960d1ea..68563186637e 100644
--- a/drivers/firmware/cirrus/cs_dsp.c
+++ b/drivers/firmware/cirrus/cs_dsp.c
@@ -2695,28 +2695,29 @@ static void cs_dsp_halo_stop_watchdog(struct cs_dsp *dsp)
}
/**
* cs_dsp_power_up() - Downloads firmware to the DSP
* @dsp: pointer to DSP structure
* @wmfw_firmware: the firmware to be sent
* @wmfw_filename: file name of firmware to be sent
- * @coeff_firmware: the coefficient data to be sent
- * @coeff_filename: file name of coefficient to data be sent
+ * @coeffs: coefficient data and filename pairs to be sent
+ * @num_coeffs: number of coefficient files to be sent
* @fw_name: the user-friendly firmware name
*
* This function is used on ADSP2 and Halo DSP cores, it powers-up the DSP core
* and downloads the firmware but does not start the firmware running. The
* cs_dsp booted flag will be set once completed and if the core has a low-power
* memory retention mode it will be put into this state after the firmware is
- * downloaded.
+ * downloaded. Differs from cs_dsp_power_up() in that it allows for multiple
+ * coefficient files to be downloaded.
*
* Return: Zero for success, a negative number on error.
*/
-int cs_dsp_power_up(struct cs_dsp *dsp,
- const struct firmware *wmfw_firmware, const char *wmfw_filename,
- const struct firmware *coeff_firmware, const char *coeff_filename,
- const char *fw_name)
+int cs_dsp_power_up_multiple(struct cs_dsp *dsp,
+ const struct firmware *wmfw_firmware, const char *wmfw_filename,
+ struct cs_dsp_coeff_desc *coeffs, int num_coeffs,
+ const char *fw_name)
{
- int ret;
+ int i, ret;
mutex_lock(&dsp->pwr_lock);
@@ -2742,9 +2743,12 @@ int cs_dsp_power_up(struct cs_dsp *dsp,
if (ret != 0)
goto err_ena;
- ret = cs_dsp_load_coeff(dsp, coeff_firmware, coeff_filename);
- if (ret != 0)
- goto err_ena;
+ for (i = 0; i < num_coeffs; i++) {
+ ret = cs_dsp_load_coeff(dsp, coeffs[i].coeff_firmware,
+ coeffs[i].coeff_filename);
+ if (ret != 0)
+ goto err_ena;
+ }
/* Initialize caches for enabled and unset controls */
ret = cs_dsp_coeff_init_control_caches(dsp);
@@ -2770,6 +2774,37 @@ int cs_dsp_power_up(struct cs_dsp *dsp,
return ret;
}
+EXPORT_SYMBOL_NS_GPL(cs_dsp_power_up_multiple, "FW_CS_DSP");
+
+/**
+ * cs_dsp_power_up() - Downloads firmware to the DSP
+ * @dsp: pointer to DSP structure
+ * @wmfw_firmware: the firmware to be sent
+ * @wmfw_filename: file name of firmware to be sent
+ * @coeff_firmware: the coefficient data to be sent
+ * @coeff_filename: file name of coefficient to data be sent
+ * @fw_name: the user-friendly firmware name
+ *
+ * This function is used on ADSP2 and Halo DSP cores, it powers-up the DSP core
+ * and downloads the firmware but does not start the firmware running. The
+ * cs_dsp booted flag will be set once completed and if the core has a low-power
+ * memory retention mode it will be put into this state after the firmware is
+ * downloaded.
+ *
+ * Return: Zero for success, a negative number on error.
+ */
+int cs_dsp_power_up(struct cs_dsp *dsp,
+ const struct firmware *wmfw_firmware, const char *wmfw_filename,
+ const struct firmware *coeff_firmware, const char *coeff_filename,
+ const char *fw_name)
+{
+ struct cs_dsp_coeff_desc coeff_desc;
+
+ coeff_desc.coeff_firmware = coeff_firmware;
+ coeff_desc.coeff_filename = coeff_filename;
+
+ return cs_dsp_power_up_multiple(dsp, wmfw_firmware, wmfw_filename, &coeff_desc, 1, fw_name);
+}
EXPORT_SYMBOL_NS_GPL(cs_dsp_power_up, "FW_CS_DSP");
/**
@@ -2695,28 +2695,29 @@ static void cs_dsp_halo_stop_watchdog(struct cs_dsp *dsp)
}
/**
- * cs_dsp_power_up() - Downloads firmware to the DSP
- * @dsp: pointer to DSP structure
+ * cs_dsp_power_up_multiple() - Downloads firmware and multiple coefficient files to the DSP
+ * @dsp: pointer to the DSP structure
* @wmfw_firmware: the firmware to be sent
* @wmfw_filename: file name of firmware to be sent
--
2.34.1
Powered by blists - more mailing lists